#!/bin/bash
#
# epever.tracert.to.domoticz.sh
#
# Tested on : EpEver Tracer 3210AN 30A MPPT Solar Controller
# Date      : 12-10-2020
# Version   : 00,00.00,000.2
#
# Created by ILoveIOT 2020
# Updated by ILoveIOT & Micha123 2022
#	- Added battery temp
#	- Added SOC value
#	- Correcting the actual voltage on the battery side
#	- Changed some small things in this script
#   - Added Real Battery Curent (Charging and Discharging) it wile be calculated from PVPOWER - LOADPOWER
#   - Fixed Batterycharging Power, now its calculating PVARRAYPOWER - LOADPOWER (bevore it shows the same as PVARRAYPOWER)
#   - Add a check if Values in $OUTPUTFILE are present, if there are no values then ihe script wil stop and dont send ZERO value to domoticz
#
####################################################################################################################################

INFO="This script will execute the /root/scripts/epevertracer/epever.tracer.php file put the output to a file, 
get the value's from the output file, show them on console if needed, and send them to Domoticz, this is super simple, easy to change.

Many thanks to Toggio https://github.com/toggio/PhpEpsolarTracer

For console use     : /root/scripts/epevertracer/epever.tracert.to.domoticz.sh
For crontab use     : /root/scripts/epevertracer/epever.tracert.to.domoticz.sh >> /dev/null 2>&1
For all the info    : php /root/scripts/epevertracer/example_cli.php

1. Change the serial port /root/scripts/epevertracer/epever.tracer.php line 26 ('/dev/ttyUSB1')
2. Change the Domoticz IP & Port (see below)
3. Add/Change the acording Domoticz IDX's (see below)
4. Optional : Change the working directory, where you wanna place the files
5. Optional : If you need more info from the controller compare epever.tracer.php vs example_cli.php
6. Optional : Maybe make 2 cronjobs, with no sunhours just poll once a hour, with sun every min."

####################################################################################################################################

# Working directory, place where all epever tracer file's are
WORKINGDIR="/root/scripts/epevertracer"

# Domoticz IP address
DOMOTICZIP="192.168.8.189"
DOMOTICZPORT="8080"

# Domoticz IDX (make a Domoticz dummy device first)
# Domoticz dummy Voltage
PVARRAYVOLTAGEIDX="286"
# Domoticz dummy 1 fase AMP
PVARRAYCURRENTIDX="287"
# Domoticz dummy Electricity (instant and counter) (change item, utility, item, change, Choose delivery and Energy read: Computed)
PVARRAYPOWERIDX="288"
# Domoticz dummy temp
PVARRAYTEMPIDX="289"
# Domoticz dummy temp
PVBATTERYTEMPIDX="312"
# Domoticz dummy Voltage
PVBATTERYVOLTAGEIDX="290"
# Domoticz dummy 1 fase AMP
BATTERYCHARGINGCURRENTIDX="291"
# Domoticz dummy Electricity (instant and counter) (change item, utility, item, change, Choose delivery and Energy read: Computed)
BATTERYCHARGINGPOWERIDX="292"
# Domoticz dummy %
BATTERYSOCIDX=""
# Domoticz dummy 1 fase AMP
BATTERYCHARGECURRENTIDX=""
# Domoticz custom sensor (Axis kWh)
GENERATEKWHTODAYIDX="294"
# Domoticz custom sensor (Axis kWh)
GENERATEKWHTOTALIDX="295"
# Domoticz custom sensor (Axis kWh)
GENERATEKWHMONTHIDX="296"
# Domoticz custom sensor (Axis kWh)
GENERATEKWHYEARIDX="297"
# Domoticz dummy 1 fase AMP
LOADCURRENTIDX="298"
# Domoticz dummy Electricity (instant and counter) (change item, utility, item, change, Energy read: Computed)
LOADPOWERIDX="306"
# Domoticz custom sensor (Axis kWh)
CONSUMEDENERGYTODAYIDX="300"
# Domoticz custom sensor (Axis kWh)
CONSUMEDENERGYTHISMONTHIDX="301"
# Domoticz custom sensor (Axis kWh)
CONSUMEDENERGYTHISYEARIDX="302"
# Domoticz custom sensor (Axis kWh)
CONSUMEDENERGYTOTALIDX="303"

####################################################################################################################################
####################################################################################################################################

# The txt output file from reading from the solar controller
OUTPUTFILE="$WORKINGDIR/epever.output.txt"

# Generate the txt file with controller output (https://github.com/toggio/PhpEpsolarTracer)
/usr/bin/php $WORKINGDIR/epever.tracer.php > $OUTPUTFILE

# Getting the value's from the output file, and make them readable, so delete the C/V/A/W/kWh letters
CHARGERTEMP=`cat $OUTPUTFILE | sed -n 14p | awk '{print $4}' | sed 's/.$//' | sed 's/.$//'`
PVBATTERYTEMP=`cat $OUTPUTFILE | sed -n 13p | awk '{print $4}' | sed 's/.$//' | sed 's/.$//'`
PVARRAYVOLTAGE=`cat $OUTPUTFILE | sed -n 4p | awk '{print $5}' | sed 's/\V//g'`
PVARRAYCURRENT=`cat $OUTPUTFILE | sed -n 5p | awk '{print $5}' | sed 's/\A//g'`
PVARRAYPOWER=`cat $OUTPUTFILE | sed -n 6p | awk '{print $5}' | sed 's/.$//'`
GENERATEKWHTODAY=`cat $OUTPUTFILE | sed -n 32p | awk '{print $5}' | sed 's/.$//' | sed 's/.$//' | sed 's/.$//'`
GENERATEKWHMONTH=`cat $OUTPUTFILE | sed -n 33p | awk '{print $6}' | sed 's/.$//' | sed 's/.$//' | sed 's/.$//'`
GENERATEKWHYEAR=`cat $OUTPUTFILE | sed -n 34p | awk '{print $6}' | sed 's/.$//' | sed 's/.$//' | sed 's/.$//'`
GENERATEKWHTOTAL=`cat $OUTPUTFILE | sed -n 35p | awk '{print $5}' | sed 's/.$//' | sed 's/.$//' | sed 's/.$//'`
BATTERYVOLTAGE=`cat $OUTPUTFILE | sed -n 7p | awk '{print $4}' | sed 's/.$//'`
BATTERYCHARGINGCURRENT=`cat $OUTPUTFILE | sed -n 8p | awk '{print $5}' | sed 's/.$//'`
BATTERYCHARGINGPOWER=`cat $OUTPUTFILE | sed -n 9p | awk '{print $5}' | sed 's/.$//'`
BATTERYSOC=`cat $OUTPUTFILE | sed -n 16p | awk '{print $4}' | sed 's/.$//'`
LOADCURRENT=`cat $OUTPUTFILE | sed -n 11p | awk '{print $4}' | sed 's/.$//'`
LOADPOWER=`cat $OUTPUTFILE | sed -n 12p | awk '{print $4}' | sed 's/.$//'`
CONSUMEDENERGYTODAY=`cat $OUTPUTFILE | sed -n 28p | awk '{print $5}' | sed 's/.$//' | sed 's/.$//' | sed 's/.$//'`
CONSUMEDENERGYTHISMONTH=`cat $OUTPUTFILE | sed -n 29p | awk '{print $6}' | sed 's/.$//' | sed 's/.$//' | sed 's/.$//'`
CONSUMEDENERGYTHISYEAR=`cat $OUTPUTFILE | sed -n 30p | awk '{print $6}' | sed 's/.$//' | sed 's/.$//' | sed 's/.$//'`
CONSUMEDENERGYTOTAL=`cat $OUTPUTFILE | sed -n 31p | awk '{print $5}' | sed 's/.$//' | sed 's/.$//' | sed 's/.$//'`

# Correcting the actual voltage on the battery side + 0.20v, you can check the difference with a multimeter and lcd display
incl=0.20
BATTERYVOLTAGECORRECTED=`echo $BATTERYVOLTAGE + $incl | bc`

#Some changes by Micha_123 (add real charging current, fix charging power to Real charging power, and add Battery SOC)
BATTERYCHARGINGPOWER="$(echo "$PVARRAYPOWER-$LOADPOWER" | bc )"
BATTERYCHARGECURRENT="$(echo  "$PVARRAYCURRENT-$LOADCURRENT" | bc)"

# Display the main value's to console with clear value's
clear
echo "$INFO"
echo ""
echo "CHARGERTEMP             = $CHARGERTEMP °C"
echo "PVBATTERYTEMP           = $PVBATTERYTEMP °C"
echo "PVARRAYVOLTAGE          = $PVARRAYVOLTAGE V"
echo "PVARRAYCURRENT          = $PVARRAYCURRENT A"
echo "PVARRAYPOWER            = $PVARRAYPOWER W"
echo "GENERATEKWHTODAY        = $GENERATEKWHTODAY kWh"
echo "GENERATEKWHMONTH        = $GENERATEKWHMONTH kWh"
echo "GENERATEKWHYEAR         = $GENERATEKWHYEAR kWh"
echo "GENERATEKWHTOTAL        = $GENERATEKWHTOTAL kWh"
echo "BATTERYVOLTAGE          = $BATTERYVOLTAGE V"
echo "BATTERYVOLTAGECORRECTED = $BATTERYVOLTAGECORRECTED V"
echo "BATTERYCHARGINGCURRENT  = $BATTERYCHARGINGCURRENT A"
echo "BATTERYCHARGINGPOWER    = $BATTERYCHARGINGPOWER W"
echo "BATTERYCHARGECURRENT    = $BATTERYCHARGECURRENT A"
echo "BATTERYSOC              = $BATTERYSOC %"
echo "LOADCURRENT             = $LOADCURRENT A"
echo "LOADPOWER               = $LOADPOWER W"
echo "CONSUMEDENERGYTODAY     = $CONSUMEDENERGYTODAY kWh"
echo "CONSUMEDENERGYTHISMONTH = $CONSUMEDENERGYTHISMONTH kWh"
echo "CONSUMEDENERGYTHISYEAR  = $CONSUMEDENERGYTHISYEAR kWh"
echo "CONSUMEDENERGYTOTAL     = $CONSUMEDENERGYTOTAL kWh"
echo ""
echo ""

# add check if values are present in $OUTPUTFILE, if there are no values nothing wil be sent to domoticz to prevent "Zero" reading in Domoticz
if [ -z "$BATTERYSOC" ]; then
	echo "No Values"
	exit 1
else
	# Some dummy value for Domoticz energy total
	DUMMYVALUE="1"
	# Send the value's to Domoticz, comment out what you're not using.
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$PVARRAYTEMPIDX&nvalue=0&svalue=$CHARGERTEMP" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$PVBATTERYTEMPIDX&nvalue=0&svalue=$PVBATTERYTEMP" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$PVARRAYVOLTAGEIDX&nvalue=0&svalue=$PVARRAYVOLTAGE" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$PVARRAYCURRENTIDX&nvalue=0&svalue=$PVARRAYCURRENT" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$PVARRAYPOWERIDX&nvalue=0&svalue="$PVARRAYPOWER";"$DUMMYVALUE"" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$PVBATTERYVOLTAGEIDX&nvalue=0&svalue=$BATTERYVOLTAGECORRECTED" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$BATTERYCHARGINGCURRENTIDX&nvalue=0&svalue=$BATTERYCHARGINGCURRENT" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$BATTERYCHARGINGPOWERIDX&nvalue=0&svalue="$BATTERYCHARGINGPOWER";"$DUMMYVALUE"" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$GENERATEKWHTODAYIDX&nvalue=0&svalue=$GENERATEKWHTODAY" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$GENERATEKWHTOTALIDX&nvalue=0&svalue=$GENERATEKWHTOTAL" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$GENERATEKWHMONTHIDX&nvalue=0&svalue=$GENERATEKWHMONTH" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$GENERATEKWHYEARIDX&nvalue=0&svalue=$GENERATEKWHYEAR" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$LOADCURRENTIDX&nvalue=0&svalue=$LOADCURRENT" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$LOADPOWERIDX&nvalue=0&svalue="$LOADPOWER";"$DUMMYVALUE"" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CONSUMEDENERGYTODAYIDX&nvalue=0&svalue=$CONSUMEDENERGYTODAY" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CONSUMEDENERGYTHISMONTHIDX&nvalue=0&svalue=$CONSUMEDENERGYTHISMONTH" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CONSUMEDENERGYTHISYEARIDX&nvalue=0&svalue=$CONSUMEDENERGYTHISYEAR" >> /dev/null 2>&1
	curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$CONSUMEDENERGYTOTALIDX&nvalue=0&svalue=$CONSUMEDENERGYTOTAL" >> /dev/null 2>&1
	#curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$BATTERYSOCIDX&nvalue=0&svalue=$BATTERYSOC" >> /dev/null 2>&1
	#curl -i -s "http://$DOMOTICZIP:$DOMOTICZPORT/json.htm?type=command&param=udevice&idx=$BATTERYCHARGECURRENTIDX&nvalue=0&svalue=$BATTERYCHARGECURRENT" >> /dev/null 2>&1
fi	
	
exit

#EOF
